"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GroupByList = void 0;
const react_1 = __importDefault(require("react"));
const react_beautiful_dnd_1 = require("react-beautiful-dnd");
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const faGripVertical_1 = require("@fortawesome/free-solid-svg-icons/faGripVertical");
const faSortUp_1 = require("@fortawesome/free-solid-svg-icons/faSortUp");
const faSortDown_1 = require("@fortawesome/free-solid-svg-icons/faSortDown");
const faSort_1 = require("@fortawesome/free-solid-svg-icons/faSort");
const GroupByList = function GroupByList({ isDragDisabled, dragColumn, groupedDirections, toggleSort, groupedColumns }) {
    const col = dragColumn === null || dragColumn === void 0 ? void 0 : dragColumn.column;
    // can't use col.isGrouped - can change before save
    const isGrouped = !!groupedColumns.find(c => c.key === (col === null || col === void 0 ? void 0 : col.key));
    const isSortable = ((col === null || col === void 0 ? void 0 : col.sortable) && col.colSpan === 1 && col.name && !isGrouped);
    return (react_1.default.createElement(react_beautiful_dnd_1.Droppable, { droppableId: 'group-by', isDropDisabled: !isSortable }, (provided, snapshot) => {
        let classNames = ['group-by-column-list'];
        if (isSortable) {
            classNames.push('is-droppable');
        }
        if (snapshot.isDraggingOver) {
            classNames.push('is-dragging-over');
        }
        return (react_1.default.createElement("div", { className: 'group-by-container' },
            react_1.default.createElement("div", null, "Group By"),
            react_1.default.createElement("div", Object.assign({ ref: provided.innerRef, className: classNames.join(' ') }, provided.droppableProps),
                groupedColumns.map((col, index) => {
                    return react_1.default.createElement(ColumnEl, { key: col.key, colIndex: index, col: col, sort: groupedDirections[index], toggleSort: toggleSort, isDragDisabled: isDragDisabled });
                }),
                provided.placeholder)));
    }));
};
exports.GroupByList = GroupByList;
const ColumnEl = function ColumnEl({ col, sort, colIndex, toggleSort, isDragDisabled }) {
    return react_1.default.createElement(react_beautiful_dnd_1.Draggable, { draggableId: 'grp-' + col.key, index: colIndex, isDragDisabled: isDragDisabled }, (provided) => (react_1.default.createElement("div", Object.assign({ className: 'config-column' }, provided.draggableProps, { ref: provided.innerRef }),
        isDragDisabled && react_1.default.createElement("div", { className: 'column-drag-placeholder' }),
        !isDragDisabled && react_1.default.createElement("div", Object.assign({ className: 'column-drag' }, provided.dragHandleProps),
            react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: faGripVertical_1.faGripVertical, fixedWidth: true })),
        react_1.default.createElement("span", { className: 'column-header' }, col.header),
        react_1.default.createElement("button", { type: 'button', title: `Sort `, className: 'toggle-sort', onClick: () => toggleSort(colIndex) },
            sort.direction === 'asc' && react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: faSortUp_1.faSortUp, className: 'sort-icon', fixedWidth: true }),
            sort.direction === 'desc' && react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: faSortDown_1.faSortDown, className: 'sort-icon', fixedWidth: true }),
            sort.direction === null && react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: faSort_1.faSort, style: { opacity: .3 }, className: 'sort-icon', fixedWidth: true })))));
};
