import React, { Component, ReactNode } from 'react';
import { TriggerEvent, StyleProps, MenuId } from '../types';
export interface MenuProps extends StyleProps {
    /**
     * Unique id to identify the menu. Use to Trigger the corresponding menu
     */
    id: MenuId;
    /**
     * Any valid node that can be rendered
     */
    children: ReactNode;
    /**
     * Theme is appended to `react-contexify__theme--${given theme}`.
     *
     * Built-in theme are `light` and `dark`
     */
    theme?: 'light' | 'dark';
    /**
     * Animation is appended to `.react-contexify__will-enter--${given animation}`
     *
     * Built-in animations are fade, flip, pop, zoom
     */
    animation?: 'fade' | 'flip' | 'pop' | 'zoom';
    /**
     * Invoked when the menu is shown.
     */
    onShown?: () => void;
    /**
     * Invoked when the menu is hidden.
     */
    onHidden?: () => void;
    /**
     * The render target of the menu
     */
    renderTarget?: () => HTMLElement | null;
    position?: 'bottom-left';
}
interface MenuState {
    x: number;
    y: number;
    visible: boolean;
    nativeEvent: TriggerEvent;
    propsFromTrigger: object;
}
export declare class Menu extends Component<MenuProps, MenuState> {
    state: {
        x: number;
        y: number;
        visible: boolean;
        nativeEvent: TriggerEvent;
        propsFromTrigger: {};
        onShown: null;
        onHidden: null;
    };
    menuRef: HTMLDivElement;
    unsub: (() => boolean)[];
    componentDidMount(): void;
    componentWillUnmount(): void;
    componentDidUpdate(_: Readonly<MenuProps>, prevState: Readonly<MenuState>): void;
    bindWindowEvent: () => void;
    unBindWindowEvent: () => void;
    onMouseEnter: () => void;
    onMouseLeave: () => void;
    hide: (event?: Event) => void;
    handleKeyboard: (e: KeyboardEvent) => void;
    setRef: (ref: HTMLDivElement) => void;
    setMenuPosition(): void;
    getMousePosition(e: TriggerEvent): {
        x: number;
        y: number;
    };
    show: (e: TriggerEvent, props: object) => void;
    render(): React.JSX.Element;
}
export {};
