"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Menu = void 0;
/* global: window */
const react_1 = __importStar(require("react"));
const cx_1 = require("../utils/cx");
const cloneItem_1 = require("./cloneItem");
const Portal_1 = require("./Portal");
const actions_1 = require("../utils/actions");
const styles_1 = require("../utils/styles");
const eventManager_1 = require("../utils/eventManager");
const firstFocusable_1 = require("../../../utils/firstFocusable");
const KEY = {
    ENTER: 13,
    ESC: 27,
    ARROW_UP: 38,
    ARROW_DOWN: 40,
    ARROW_LEFT: 37,
    ARROW_RIGHT: 39
};
class Menu extends react_1.Component {
    constructor() {
        super(...arguments);
        this.state = {
            x: 0,
            y: 0,
            visible: false,
            nativeEvent: {},
            propsFromTrigger: {},
            onShown: null,
            onHidden: null
        };
        this.unsub = [];
        this.bindWindowEvent = () => {
            window.addEventListener('resize', this.hide);
            window.addEventListener('contextmenu', this.hide);
            window.addEventListener('mousedown', this.hide);
            window.addEventListener('click', this.hide);
            window.addEventListener('scroll', this.hide);
            window.addEventListener('keydown', this.handleKeyboard);
        };
        this.unBindWindowEvent = () => {
            window.removeEventListener('resize', this.hide);
            window.removeEventListener('contextmenu', this.hide);
            window.removeEventListener('mousedown', this.hide);
            window.removeEventListener('click', this.hide);
            window.removeEventListener('scroll', this.hide);
            window.removeEventListener('keydown', this.handleKeyboard);
        };
        this.onMouseEnter = () => window.removeEventListener('mousedown', this.hide);
        this.onMouseLeave = () => window.addEventListener('mousedown', this.hide);
        this.hide = (event) => {
            // Safari trigger a click event when you ctrl + trackpad
            // Firefox:  trigger a click event when right click occur
            const e = event;
            if (typeof e !== 'undefined' &&
                (e.button === 2 || e.ctrlKey === true) &&
                e.type !== 'contextmenu') {
                return;
            }
            this.unBindWindowEvent();
            this.setState({ visible: false });
        };
        this.handleKeyboard = (e) => {
            if (e.keyCode === KEY.ENTER || e.keyCode === KEY.ESC) {
                e.stopPropagation();
                this.unBindWindowEvent();
                this.setState({ visible: false });
            }
        };
        this.setRef = (ref) => {
            this.menuRef = ref;
        };
        this.show = (e, props) => {
            e.stopPropagation();
            eventManager_1.eventManager.emit(actions_1.HIDE_ALL);
            const { x, y } = this.getMousePosition(e);
            this.setState({
                visible: true,
                x,
                y,
                nativeEvent: e,
                propsFromTrigger: props
            }, this.setMenuPosition);
        };
    }
    componentDidMount() {
        this.unsub.push(eventManager_1.eventManager.on((0, actions_1.DISPLAY_MENU)(this.props.id), this.show));
        this.unsub.push(eventManager_1.eventManager.on(actions_1.HIDE_ALL, this.hide));
    }
    componentWillUnmount() {
        this.unsub.forEach(cb => cb());
        this.unBindWindowEvent();
    }
    componentDidUpdate(_, prevState) {
        if (this.state.visible !== prevState.visible) {
            if (this.state.visible && this.menuRef) {
                let firstFocusableEl = (0, firstFocusable_1.firstFocusable)(this.menuRef);
                firstFocusableEl === null || firstFocusableEl === void 0 ? void 0 : firstFocusableEl.focus();
            }
            else if (!this.state.visible) {
                let menuProviderEl = document.getElementById(this.props.id);
                menuProviderEl === null || menuProviderEl === void 0 ? void 0 : menuProviderEl.focus();
            }
            if (this.state.visible && this.props.onShown) {
                this.props.onShown();
            }
            else if (!this.state.visible && this.props.onHidden) {
                this.props.onHidden();
            }
        }
    }
    setMenuPosition() {
        const { innerWidth: windowWidth, innerHeight: windowHeight } = window;
        const { offsetWidth: menuWidth, offsetHeight: menuHeight } = this.menuRef;
        let { x, y } = this.state;
        if (x + menuWidth > windowWidth) {
            x -= x + menuWidth - windowWidth;
        }
        if (y + menuHeight > windowHeight) {
            y -= y + menuHeight - windowHeight;
        }
        this.setState({
            x,
            y
        }, this.bindWindowEvent);
    }
    getMousePosition(e) {
        if (this.props.position) {
            let el = document.getElementById(this.props.id);
            if (el) {
                let elRect = el.getBoundingClientRect();
                return {
                    x: elRect.left < 0 ? 0 : elRect.left,
                    y: elRect.bottom < 0 ? 0 : elRect.bottom,
                };
            }
        }
        const pos = {
            x: e.clientX,
            y: e.clientY
        };
        if (e.type === 'touchend' &&
            (!pos.x || !pos.y) &&
            (e.changedTouches && e.changedTouches.length > 0)) {
            pos.x = e.changedTouches[0].clientX;
            pos.y = e.changedTouches[0].clientY;
        }
        if (!pos.x || pos.x < 0) {
            pos.x = 0;
        }
        if (!pos.y || pos.y < 0) {
            pos.y = 0;
        }
        return pos;
    }
    render() {
        const { theme, animation, style, className, children, renderTarget } = this.props;
        const { visible, nativeEvent, propsFromTrigger, x, y } = this.state;
        const cssClasses = (0, cx_1.cx)(styles_1.styles.menu, className, {
            [styles_1.styles.theme + theme]: theme,
            [styles_1.styles.animationWillEnter + animation]: animation
        });
        const menuStyle = Object.assign(Object.assign({}, style), { left: x, top: y + 1, opacity: 1 });
        return (react_1.default.createElement(Portal_1.Portal, { renderTarget: renderTarget }, visible && (react_1.default.createElement("div", { className: cssClasses, style: menuStyle, ref: this.setRef, onMouseEnter: this.onMouseEnter, onMouseLeave: this.onMouseLeave },
            react_1.default.createElement("div", null, (0, cloneItem_1.cloneItem)(children, {
                nativeEvent,
                propsFromTrigger
            }))))));
    }
}
exports.Menu = Menu;
