"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterGroupEditor = void 0;
const react_1 = __importStar(require("react"));
const contexts_1 = require("../../table/contexts");
const types_1 = require("../../table/types");
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const faPlusCircle_1 = require("@fortawesome/free-solid-svg-icons/faPlusCircle");
const field_1 = require("./field");
const context_menu_1 = require("../../context-menu");
const closest_1 = require("../../../utils/closest");
const useDerivedState_1 = require("../../../utils/useDerivedState");
const helpers_1 = require("../helpers");
const FilterGroupEditor = function FilterGroupEditor({ value, path, setState }) {
    const { filterSettings, filterColumns } = (0, react_1.useContext)(contexts_1.ColumnContext);
    const groupEl = (0, react_1.useRef)(null);
    const [{ allowNested, allowOr, canAddCondition, canLimitConditions, availableColumns, }] = (0, useDerivedState_1.useDerivedState)(() => {
        var _a, _b, _c;
        const filterableColumns = filterColumns.filter(c => !!c.filter);
        const allowOr = (_a = filterSettings === null || filterSettings === void 0 ? void 0 : filterSettings.allowOr) !== null && _a !== void 0 ? _a : false;
        const allowNested = (_b = filterSettings === null || filterSettings === void 0 ? void 0 : filterSettings.allowNested) !== null && _b !== void 0 ? _b : false;
        const limitOneColumnUse = (_c = filterSettings === null || filterSettings === void 0 ? void 0 : filterSettings.limitOneColumnUse) !== null && _c !== void 0 ? _c : false;
        const canLimitConditions = (!allowNested && limitOneColumnUse);
        let canAddCondition = true;
        let availableColumns = filterableColumns;
        if (canLimitConditions) {
            canAddCondition = value.filters.length < filterableColumns.length;
            availableColumns = filterableColumns.filter(c => {
                return !value.filters.find(f => {
                    if ((0, types_1.isFilterGroup)(f))
                        return false;
                    return c.filter.filterKey === f.column;
                });
            });
        }
        return {
            allowNested,
            allowOr,
            canAddCondition,
            canLimitConditions,
            availableColumns,
        };
    }, [filterColumns, filterSettings, value]);
    const currentPath = path !== null && path !== void 0 ? path : [];
    const currentPathAsString = currentPath.join(',');
    function setGroupCondition(operator) {
        setState(currentPath, { groupOperator: { $set: operator } });
    }
    function addGroup() {
        setState(currentPath, { filters: { $push: [{ groupOperator: 'and', filters: [] }] } });
    }
    function removeGroup() {
        if (!currentPath.length) {
            // top-level
            setState(currentPath, { filters: { $set: [] } });
        }
        else {
            let upPath = [...currentPath];
            let lastIdx = upPath.pop();
            setState(upPath, { filters: { $splice: [[lastIdx, 1]] } });
        }
    }
    function addCondition() {
        // get first available column and set it
        let column = availableColumns === null || availableColumns === void 0 ? void 0 : availableColumns[0];
        if (!column) {
            console.error('Failed to find available column to add as default condition');
            return;
        }
        let defaultOp = (0, helpers_1.getDefaultOperator)(column.filter);
        setState(currentPath, {
            filters: { $push: [{
                        column: column.filter.filterKey,
                        value: (0, helpers_1.getDefaultValue)(defaultOp, column.filter),
                        operator: defaultOp,
                    }] }
        });
    }
    return react_1.default.createElement("div", { ref: groupEl, className: 'filter-group' },
        react_1.default.createElement("div", { className: 'filter-group-options' },
            (allowNested || allowOr) && react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(context_menu_1.MenuProvider, { id: `group_menu_${currentPathAsString}`, className: `filter-group-operator filter-group-operator-${value.groupOperator}`, event: 'onClick' },
                    value.groupOperator === 'and' && 'And',
                    value.groupOperator === 'or' && 'Or'),
                react_1.default.createElement(context_menu_1.Menu, { id: `group_menu_${currentPathAsString}`, renderTarget: () => (0, closest_1.closest)(groupEl.current, 'dialog, .dialog'), position: 'bottom-left', animation: 'zoom' },
                    allowOr && react_1.default.createElement(react_1.default.Fragment, null,
                        react_1.default.createElement(context_menu_1.Item, { onClick: () => setGroupCondition('and') }, "And"),
                        react_1.default.createElement(context_menu_1.Item, { onClick: () => setGroupCondition('or') }, "Or"),
                        (allowNested || canAddCondition) && react_1.default.createElement(context_menu_1.Separator, null)),
                    allowNested && react_1.default.createElement(context_menu_1.Item, { onClick: () => addGroup() }, "Add Group"),
                    canAddCondition && react_1.default.createElement(context_menu_1.Item, { onClick: () => addCondition() }, "Add Condition"),
                    allowNested && react_1.default.createElement(react_1.default.Fragment, null,
                        react_1.default.createElement(context_menu_1.Separator, null),
                        react_1.default.createElement(context_menu_1.Item, { onClick: () => removeGroup() }, "Remove")))),
            canAddCondition && react_1.default.createElement("button", { type: 'button', className: 'filter-group-add-btn filter-btn', onClick: () => addCondition() },
                react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: faPlusCircle_1.faPlusCircle }))),
        react_1.default.createElement("div", { className: 'filter-group-filters' }, value.filters.map((filter, idx) => {
            let newPath = [...currentPath, idx];
            return (0, types_1.isFilterGroup)(filter)
                ? react_1.default.createElement(FilterGroupEditor, { key: newPath.join(','), setState: setState, value: filter, path: newPath })
                : react_1.default.createElement(field_1.FilterFieldEditor, { key: newPath.join(','), setState: setState, groupValue: value, filter: filter, path: currentPath, index: idx, canLimitConditions: canLimitConditions });
        })));
};
exports.FilterGroupEditor = FilterGroupEditor;
