"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterValueEditor = void 0;
const react_1 = __importDefault(require("react"));
const types_1 = require("../../../table/types");
const input_1 = require("./input");
const react_fontawesome_1 = require("@fortawesome/react-fontawesome");
const faPlusCircle_1 = require("@fortawesome/free-solid-svg-icons/faPlusCircle");
const faTimesCircle_1 = require("@fortawesome/free-solid-svg-icons/faTimesCircle");
const boolean_1 = require("./boolean");
const custom_1 = require("./custom");
const FilterValueEditor = function FilterValueEditor(props) {
    const { filter: { operator } } = props;
    switch (operator) {
        case 'nul':
        case 'nnul':
            return react_1.default.createElement(react_1.default.Fragment, null); // Null checks don't have values
        case 'bet':
        case 'nbet':
            return react_1.default.createElement(DoubleValueEditor, Object.assign({}, props));
        case 'any':
        case 'none':
            return react_1.default.createElement(MultiValueEditor, Object.assign({}, props));
        default:
            return react_1.default.createElement(SingleValueEditor, Object.assign({}, props));
    }
};
exports.FilterValueEditor = FilterValueEditor;
const SingleValueEditor = function SingleValueEditor(_a) {
    var { column, valuePath } = _a, rest = __rest(_a, ["column", "valuePath"]);
    if (!column)
        return react_1.default.createElement("span", { className: 'filter-editor-value' },
            react_1.default.createElement("span", { className: 'no-value' }, "<Choose Column>"));
    switch (column.filter.type) {
        case 'email':
        case 'number':
        case 'string':
            let actualType = column.filter.type;
            if (actualType === 'string')
                actualType = 'text';
            return react_1.default.createElement(input_1.InputEditor, Object.assign({ type: actualType, valuePath: valuePath !== null && valuePath !== void 0 ? valuePath : null }, rest));
        case 'boolean':
            return react_1.default.createElement(boolean_1.BooleanEditor, Object.assign({ type: column.filter.type, valuePath: valuePath !== null && valuePath !== void 0 ? valuePath : null }, rest));
        case 'custom':
            return react_1.default.createElement(custom_1.CustomEditor, Object.assign({ filterDef: column.filter, type: column.filter.type, valuePath: valuePath !== null && valuePath !== void 0 ? valuePath : null }, rest));
        default:
            return react_1.default.createElement("span", { className: 'filter-editor-value' },
                react_1.default.createElement("span", { className: 'no-value' }, "<Unknown Filter Type>"));
    }
};
const DoubleValueEditor = function DoubleValueEditor(_a) {
    var { column } = _a, rest = __rest(_a, ["column"]);
    if (!column)
        return react_1.default.createElement("span", { className: 'filter-editor-value' },
            react_1.default.createElement("span", { className: 'no-value' }, "<Choose Column>"));
    return react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(SingleValueEditor, Object.assign({ column: column, valuePath: 0 }, rest)),
        react_1.default.createElement("span", { className: 'double-value-separator' }, "and"),
        react_1.default.createElement(SingleValueEditor, Object.assign({ column: column, valuePath: 1 }, rest)));
};
const MultiValueEditor = function MultiValueEditor(_a) {
    var _b;
    var { column, filter, setState } = _a, rest = __rest(_a, ["column", "filter", "setState"]);
    if (!column)
        return react_1.default.createElement("span", { className: 'filter-editor-value' },
            react_1.default.createElement("span", { className: 'no-value' }, "<Choose Column>"));
    function addValue() {
        const { path, index } = rest;
        setState(path, {
            filters: {
                [index]: {
                    value: { $push: [null] }
                }
            }
        });
    }
    function removeValue(idxToRemove) {
        const { path, index } = rest;
        setState(path, {
            filters: {
                [index]: {
                    value: { $splice: [[idxToRemove, 1]] }
                }
            }
        });
    }
    let values = (_b = filter.value) !== null && _b !== void 0 ? _b : [];
    let { value: noValues } = filter, filterWithoutValues = __rest(filter, ["value"]);
    let valueKeyBase = JSON.stringify(Object.assign(Object.assign({}, filterWithoutValues), { path: rest.path, idx: rest.index }));
    if ((0, types_1.isCustomFilter)(column.filter) && !!column.filter.MultiEditor) {
        return react_1.default.createElement(custom_1.CustomEditor, Object.assign({ useMulti: true, filterDef: column.filter, setState: setState, filter: filter, type: column.filter.type, valuePath: null }, rest));
    }
    return react_1.default.createElement(react_1.default.Fragment, null,
        "(",
        values.map((_value, index, arr) => (react_1.default.createElement("span", { className: 'multi-value-editor', key: valueKeyBase + '_' + index },
            react_1.default.createElement(SingleValueEditor, Object.assign({ column: column, setState: setState, valuePath: index, filter: filter }, rest)),
            arr.length > 1 && react_1.default.createElement("button", { type: 'button', "data-noautofocus": true, className: 'filter-mv-item-remove filter-btn', onClick: () => removeValue(index) },
                react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: faTimesCircle_1.faTimesCircle }))))),
        react_1.default.createElement("button", { type: 'button', "data-noautofocus": true, className: 'filter-mv-item-add filter-btn', onClick: addValue },
            react_1.default.createElement(react_fontawesome_1.FontAwesomeIcon, { icon: faPlusCircle_1.faPlusCircle })),
        ")");
};
