"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.valueShouldBeArray = exports.ValueCount = exports.getDefaultValue = exports.getDefaultOperator = exports.getAvailableOperators = void 0;
const types_1 = require("../table/types");
function getAvailableOperators(filter) {
    var _a;
    if (!filter)
        return [];
    if ((_a = filter.operators) === null || _a === void 0 ? void 0 : _a.length)
        return filter.operators;
    switch (filter.type) {
        case 'string':
        case 'email':
            return [...types_1.StringOperators];
        case 'number':
            return [...types_1.NumberOperators];
        case 'boolean':
            return [...types_1.BooleanOperators];
        case 'custom':
            return Array.from(new Set([...types_1.StringOperators, ...types_1.NumberOperators, ...types_1.BooleanOperators]));
    }
}
exports.getAvailableOperators = getAvailableOperators;
function getDefaultOperator(filter) {
    var _a, _b;
    let available = getAvailableOperators(filter);
    return (_b = (_a = filter === null || filter === void 0 ? void 0 : filter.defaultOperator) !== null && _a !== void 0 ? _a : available === null || available === void 0 ? void 0 : available[0]) !== null && _b !== void 0 ? _b : 'eq';
}
exports.getDefaultOperator = getDefaultOperator;
function getDefaultValue(op, filter) {
    var _a;
    if (!filter)
        return null;
    if (filter.type === 'custom') {
        return (_a = filter.defaultValue) !== null && _a !== void 0 ? _a : null;
    }
    switch (filter.type) {
        case 'string':
        case 'email':
            return convertDefaultValueToArray(op, '');
        case 'number':
            return convertDefaultValueToArray(op, Number.NaN);
        case 'boolean':
            return convertDefaultValueToArray(op, false);
    }
    return null;
}
exports.getDefaultValue = getDefaultValue;
var ValueCount;
(function (ValueCount) {
    ValueCount[ValueCount["SingleValue"] = 0] = "SingleValue";
    ValueCount[ValueCount["DualValue"] = 1] = "DualValue";
    ValueCount[ValueCount["ManyValue"] = 2] = "ManyValue";
    ValueCount[ValueCount["NoValue"] = 3] = "NoValue";
})(ValueCount = exports.ValueCount || (exports.ValueCount = {}));
function valueShouldBeArray(operator) {
    switch (operator) {
        case 'nul':
        case 'nnul':
            return ValueCount.NoValue;
        case 'bet':
        case 'nbet':
            return ValueCount.DualValue;
        case 'any':
        case 'none':
            return ValueCount.ManyValue;
        default:
            return ValueCount.SingleValue;
    }
}
exports.valueShouldBeArray = valueShouldBeArray;
function convertDefaultValueToArray(operator, value) {
    const arrayType = valueShouldBeArray(operator);
    if (arrayType === ValueCount.NoValue)
        return null;
    else if (arrayType === ValueCount.ManyValue)
        return [null];
    else if (arrayType === ValueCount.DualValue)
        return [null, null];
    return value;
}
