import React, { DetailedHTMLProps, TdHTMLAttributes, HTMLAttributes } from 'react';
import { DataColumn, ColumnSorts, ColumnSort, OnShowColumnPicker, QueryFilterGroup, EditFormData, CustomClasses, CustomLabels, CustomComponents, SetFilterCb, SetPaginationCb, OnShowFilterEditor, ColumnConfigurationWithGroup, EditModes, OnSaveQuickEdit } from './types';
import { FilterSettings } from '../filter/types';
export interface ColumnContextInterface<T> {
    preMDRColumn?: DataColumn<T>;
    actualColumns: DataColumn<T>[];
    filterColumns: DataColumn<T>[];
    headerRows: DataColumn<T>[][];
    columnSorts: ColumnSort[];
    groupBy: ColumnSort[];
    filter: QueryFilterGroup;
    filterSettings?: FilterSettings;
    multiColumnSorts: boolean;
    canSelectRows: boolean;
    canGroupBy: boolean;
    setColumnSort: (newState: ColumnSorts | ((state: ColumnSorts) => ColumnSorts)) => void;
    setAllSelected: (selectAll: boolean, data: T[]) => void;
    setRowSelected: (row: T, rowIndex: number) => void;
    onShowColumnPicker?: OnShowColumnPicker;
    onShowFilterEditor?: OnShowFilterEditor;
    setFilter: SetFilterCb;
    setPagination: SetPaginationCb;
    getRowKey?: (row: T) => string | number;
    propOnSaveQuickEdit?: OnSaveQuickEdit<T>;
    DetailRow?: React.ElementType<{
        parentRow: T;
    }>;
    canRowShowDetail?: (row: T) => boolean;
    columnOrder: string[];
    setColumnConfig: (config: ColumnConfigurationWithGroup) => void;
    canReorderColumns: boolean;
    canSelectRow?: (row: T) => boolean;
    classNames?: CustomClasses;
    labels?: CustomLabels;
    components?: CustomComponents<T>;
    groupsExpandedByDefault: boolean;
    doNotUseHTML5Dialog?: boolean;
    doNotUseRefetchAfterSave: boolean;
    getTableRowProps?: (row: T) => (DetailedHTMLProps<HTMLAttributes<HTMLTableRowElement>, HTMLTableRowElement> | null | undefined);
    getTableCellProps?: (value: any, row: T, column: DataColumn<T>) => (DetailedHTMLProps<TdHTMLAttributes<HTMLTableCellElement>, HTMLTableCellElement> | null | undefined);
}
export declare const ColumnContext: React.Context<ColumnContextInterface<any>>;
interface GroupCollapseContextInterface {
    setExpanded: (groupKey: string, expanded: boolean) => void;
    collapsedState: Record<string, boolean>;
}
export declare const GroupCollapseContext: React.Context<GroupCollapseContextInterface>;
export interface TableContextInterface<T> {
    isEditing: boolean;
    isSavingQuickEdit: boolean;
    editData: EditFormData;
    editMode: EditModes;
    selectedRows: Record<string | number, T>;
}
export declare const TableContext: import("@borvik/react-selector-context/dist/types").SelectorContext<TableContextInterface<any>>;
interface TableContextProviderProps {
    editMode: EditModes;
    children?: React.ReactNode;
}
export declare const useTableContexSetter: () => import("@borvik/react-selector-context/dist/types").UpdaterCallback<TableContextInterface<any>>;
export declare const useTableSelector: import("@borvik/react-selector-context/dist/types").UseStateHookDef<TableContextInterface<any>>;
export declare const TableContextProvider: React.FC<TableContextProviderProps>;
export {};
