"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableDataProvider = exports.DataProviderContent = exports.TableDataContext = void 0;
const react_1 = __importStar(require("react"));
const useDeepEffect_1 = require("../../utils/useDeepEffect");
const useDeepMemo_1 = require("../../utils/useDeepMemo");
const useDerivedState_1 = require("../../utils/useDerivedState");
const contexts_1 = require("./contexts");
const EMPTY_ARRAY = [];
exports.TableDataContext = (0, react_1.createContext)({
    data: [],
    loading: false,
    refetch: () => { },
});
/*

data: [] -> direct and any changes should rerender, works with footerData, total and isLoading
data: async () => [] -> result of call should rerender, no total/footerData - but manages isLoading
data: async () => {data,footer,total} -> similar to previous, but has total/footerData
data: async () => React.Element -> must render element - which should call setData to trigger re-renders

components.DataProvider: React.Element
used like
<DataTable
  components={{
    DataProvider - default provider does above
  }}
/>

const MyProvider = ({ filters, pagination, sorts, search, BODY, children }) => {
  let apiFilters: any = undefined;
  if (filters) {
    apiFilters = translateTableFilter(filters); // turn to hook? (for rerender purposes)
  }
  return <Query
    query={gql``}
    variables={{
      filters: apiFilters
    }}
  >
    {({data, loading}) => {
      <BODY
        data={data.qry.list ?? []}
        total={data.qry.total ?? undefined}
        footerData={data.qry.footer}
        isLoading={loading}
        children={children}
      />
    }}
  </Query>
};


so... we need multiple levels here.

main table component needs a generic provider to wrap a number of it's children
the provider is given a BODY component and the table children which is itself the
  _real_ data provider
  the children include stuff like the main table rendering elements
  this must _also_ be passed to the _real_ provider for rendering

*/
const DataProviderContent = function DataProviderContent({ loading, dataError, data, footerData, total, refetch, children, }) {
    // Using `useDeepMemo` to ensure same instance
    const value = (0, useDeepMemo_1.useDeepMemo)(() => {
        return {
            data,
            dataError,
            footerData,
            total,
            loading,
            refetch,
        };
    }, [loading, total, dataError, refetch, footerData, data]);
    return react_1.default.createElement(exports.TableDataContext.Provider, { value: value }, children);
};
exports.DataProviderContent = DataProviderContent;
const TableDataProvider = function TableDataProvider(_a) {
    var { children, TableBody } = _a, tableArgs = __rest(_a, ["children", "TableBody"]);
    const _b = useTableData(tableArgs), { loaderElement, dataNotSuppliedError } = _b, tableData = __rest(_b, ["loaderElement", "dataNotSuppliedError"]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        loaderElement,
        react_1.default.createElement(TableBody, Object.assign({}, tableData, { dataError: dataNotSuppliedError ? 'No data was supplied to the data table - either define a data prop, or a custom DataProvider' : undefined }), children)));
};
exports.TableDataProvider = TableDataProvider;
function useTableData({ passColumnsToQuery, pagination, searchQuery, onQueryChange, data: propData, totalCount, isLoading: propIsLoading, footerData, refetch: propRefetch, canGroupBy, filters, sorts, }) {
    var _a;
    const { actualColumns, setAllSelected, } = (0, react_1.useContext)(contexts_1.ColumnContext);
    const needsLoader = (typeof propData === 'function');
    const propDataComparator = typeof propData === 'function' ? 'function' : propData;
    const [data, setData] = (0, useDerivedState_1.useDerivedState)(() => {
        var _a;
        return ({
            data: (!propData || typeof propData === 'function') ? EMPTY_ARRAY : propData,
            total: (_a = ((typeof propData === 'function') ? undefined : totalCount !== null && totalCount !== void 0 ? totalCount : propData === null || propData === void 0 ? void 0 : propData.length)) !== null && _a !== void 0 ? _a : 0,
            footerData,
            refetch: (typeof propData === 'function') ? undefined : propRefetch,
        });
    }, [totalCount, propRefetch, footerData, propDataComparator]);
    const [isLoading, setIsLoading] = (0, useDerivedState_1.useDerivedState)(() => needsLoader ? true : (propIsLoading !== null && propIsLoading !== void 0 ? propIsLoading : false), [needsLoader, propIsLoading]);
    const [loaderElement, setLoaderElement] = (0, react_1.useState)(null);
    const [rerenderCount, forceRender] = (0, react_1.useState)(1);
    const doSetData = (0, react_1.useCallback)((data) => {
        if (Array.isArray(data)) {
            setData({ data, total: data.length });
        }
        else {
            setData(data);
        }
        setIsLoading(false);
    }, [setData, setIsLoading]);
    // Not really "visible" columns - but columns that _could_ be part of the query
    const visibleColumnComparator = passColumnsToQuery ? actualColumns : false;
    (0, useDeepEffect_1.useDeepEffect)(() => {
        function getData() {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof onQueryChange === 'function') {
                    onQueryChange({
                        pagination,
                        search: searchQuery !== null && searchQuery !== void 0 ? searchQuery : '',
                        sorts,
                        filters: filters.filters.length ? filters : undefined,
                        visibleColumns: actualColumns.filter(c => c.isVisible),
                    });
                }
                if (typeof propData === 'function') {
                    let returnedData = yield propData({
                        pagination,
                        search: searchQuery !== null && searchQuery !== void 0 ? searchQuery : '',
                        sorts,
                        filters: filters.filters.length ? filters : undefined,
                        visibleColumns: actualColumns.filter(c => c.isVisible),
                    }, doSetData);
                    if (react_1.default.isValidElement(returnedData)) {
                        setLoaderElement(returnedData);
                    }
                    else {
                        doSetData(returnedData);
                    }
                }
            });
        }
        if (typeof propData === 'function') {
            setIsLoading(true);
        }
        setAllSelected(false, []);
        // TODO: clear edit data (on filter/page/search)?
        getData();
    }, [rerenderCount, canGroupBy, pagination, searchQuery, sorts, filters, propDataComparator, visibleColumnComparator]);
    /**
     * Refetch... priority
     *
     * 1. props.refetch (saved to data from derived state)
     * 2. callback refetch (saved to data from doSetData)
     * 3. if no refetch - use one that will toggle the effect and run getData
     */
    const fallbackRefetch = (0, react_1.useCallback)(() => { forceRender(v => 0 - v); }, [forceRender]);
    return {
        loading: isLoading,
        data: data.data,
        footerData: data.footerData,
        total: data.total,
        refetch: (_a = data.refetch) !== null && _a !== void 0 ? _a : fallbackRefetch,
        loaderElement,
        dataNotSuppliedError: !propData,
    };
}
