"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataTable = void 0;
const react_1 = __importStar(require("react"));
const react_dom_1 = __importDefault(require("react-dom"));
const useDerivedState_1 = require("../../utils/useDerivedState");
const use_querystate_1 = require("@borvik/use-querystate");
const transformColumnProps_1 = require("../../utils/transformColumnProps");
const useLocalState_1 = require("../../utils/useLocalState");
const contexts_1 = require("./contexts");
const header_1 = require("./header");
const body_1 = require("./body");
const pagination_1 = require("../pagination");
const search_1 = require("../search");
const useParsedQS_1 = require("../../utils/useParsedQS");
const comparators_1 = require("../../utils/comparators");
const column_picker_1 = require("../column-picker");
const filter_1 = require("../filter");
const transformFilter_1 = require("../../utils/transformFilter");
const editButton_1 = require("./editors/editButton");
const actions_1 = require("./actions");
const getRowKey_1 = require("../../utils/getRowKey");
const immutable_1 = require("../../utils/immutable");
const querystring_1 = require("@borvik/querystring");
const footer_1 = require("./footer");
const contexts_2 = require("./contexts");
const data_provider_1 = require("./data-provider");
const preMDR_RenderWarned = {};
const preMDR_WidthWarned = {};
const primaryKeyWarned = {};
const fixedLeftWarned = {};
const fixedRightWarned = {};
const DataTableCore = function DataTableCore(_a) {
    var _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11;
    var { paginate = 'both', quickEditPosition = 'both', hideSearchForm = false, defaultFilter, methodRef, passColumnsToQuery } = _a, props = __rest(_a, ["paginate", "quickEditPosition", "hideSearchForm", "defaultFilter", "methodRef", "passColumnsToQuery"]);
    const canGroupBy = !!props.canGroupBy && !!props.multiColumnSorts;
    /**
     * First let's get the user-defined column visibility
     *
     * The first two values are the initializers the key the value _may_
     * be found at, and the default value if not found.
     *
     * As long as props.id doesn't change this should always be the same
     * object.
     *
     * setColumnVisibility allows for changing the value just like a
     * normal set function from `useState`, but here it also stores it
     * in localStorage at the key specified.
     */
    const [columnVisibility, setColumnVisibility] = (0, useLocalState_1.useLocalState)(`table.${props.id}.columns`, {}, [props.id]);
    const [columnOrder, setColumnOrder] = (0, useLocalState_1.useLocalState)(`table.${props.id}.columnOrder`, [], [props.id]);
    const [qsGroupBy, setGroupBy] = (0, useParsedQS_1.useParsedQs)({ group: (_b = props.defaultGroupBy) !== null && _b !== void 0 ? _b : [] }, (qsSort) => {
        var _a;
        return ({
            group: ((_a = qsSort === null || qsSort === void 0 ? void 0 : qsSort.group) !== null && _a !== void 0 ? _a : []).map(v => {
                let parts = v.split(' ').filter(a => !!a);
                if (parts.length !== 2)
                    return null;
                return {
                    column: parts[0],
                    direction: (parts[1].toLowerCase() === 'desc' ? 'desc' : 'asc')
                };
            })
                .filter(comparators_1.notEmpty)
        });
    }, (state) => {
        var _a;
        if (!state.group.length && ((_a = props.defaultGroupBy) === null || _a === void 0 ? void 0 : _a.length)) {
            return { group: [''] };
        }
        return {
            group: state.group.map(v => `${v.column} ${v.direction}`)
        };
    }, Object.assign(Object.assign({}, props.qs), { types: {
            group: 'string[]'
        } }));
    const groupBy = canGroupBy
        ? qsGroupBy.group
        : [];
    const setColumnConfig = (0, react_1.useCallback)((config) => {
        react_dom_1.default.unstable_batchedUpdates(() => {
            setColumnVisibility(config.visibility);
            setColumnOrder(config.columnOrder);
            setGroupBy({ group: config.groupBy });
        });
    }, [setColumnVisibility, setColumnOrder, setGroupBy]);
    /**
     * Using a deep comparison get the memoized column data.
     *
     * This function is "memoized" to help keep the object reference
     * consistent - not necessarily to help with performance of a recursive
     * function (since deep compare needs recursive as well).
     *
     * This is all necessary because of the columns prop.
     * If a `const` columns property is passed - it's not really necessary,
     * but if the more likely scenario is that the column definition is built
     * when the component is used - its a new reference every time.
     */
    const [columnData] = (0, useDerivedState_1.useDeepDerivedState)(() => {
        // First Clean the columns - transforms "resolve | type" column properties
        let preMDRColumn = props.preMDRColumn ? (0, transformColumnProps_1.transformColumns)(props.id, [props.preMDRColumn], columnVisibility, groupBy) : null;
        let visibleColumns = (0, transformColumnProps_1.transformColumns)(props.id, props.columns, columnVisibility, groupBy);
        let actualColumns = (0, transformColumnProps_1.getFlattenedColumns)(visibleColumns);
        let headerRows = (0, transformColumnProps_1.generateHeaderRows)(actualColumns, props.canReorderColumns ? columnOrder : []);
        let primaryKeyCount = 0, fixedLeftCount = 0, fixedRightCount = 0, hasEditor = false;
        if (preMDRColumn === null || preMDRColumn === void 0 ? void 0 : preMDRColumn[0]) {
            if (!preMDR_RenderWarned[props.id] && typeof preMDRColumn[0].render !== 'function') {
                console.warn(`Pre-MDR Column needs to have a render function defined`);
                preMDR_RenderWarned[props.id] = true;
            }
            if (!preMDR_WidthWarned[props.id] && typeof preMDRColumn[0].preMDRColumnWidth !== 'number') {
                console.warn(`Pre-MDR Column needs a width defined as a number`);
                preMDR_WidthWarned[props.id] = true;
            }
        }
        if (!props.getRowKey) {
            for (let c of actualColumns) {
                if (c.isPrimaryKey) {
                    primaryKeyCount++;
                    // Only warn once per table
                    if (primaryKeyCount > 1 && !primaryKeyWarned[props.id]) {
                        console.warn(`Primary Key defined twice - using first primary key`);
                        primaryKeyWarned[props.id] = true;
                    }
                }
                if (c.editor)
                    hasEditor = true;
            }
        }
        if (!props.suppressFixedWarning && !fixedRightWarned[props.id] && !fixedLeftWarned[props.id]) {
            for (let c of actualColumns) {
                if (c.fixed === 'left') {
                    fixedLeftCount++;
                    // TODO: if more than 1 fixed column becomes supported - adjust left/right filters in column-picker/dialog to filter out first/last "hidden" columns respectively
                    if (fixedLeftCount > 1 && !fixedLeftWarned[props.id]) {
                        console.warn(`Default styles only support 1 fixed left column`);
                        fixedLeftWarned[props.id] = true;
                    }
                }
                if (c.fixed === 'right') {
                    fixedRightCount++;
                    if (fixedRightCount > 1 && !fixedRightWarned[props.id]) {
                        console.warn(`Default styles only support 1 fixed right column`);
                        fixedRightWarned[props.id] = true;
                    }
                }
            }
        }
        // Now format the columns for easier use, and return as derived state
        return {
            headerRows,
            actualColumns,
            hasEditor,
            primaryKeyCount,
            preMDRColumn: preMDRColumn === null || preMDRColumn === void 0 ? void 0 : preMDRColumn[0]
        };
    }, [props.id, columnVisibility, props.preMDRColumn, props.columns, columnOrder, groupBy, props.canReorderColumns, canGroupBy]);
    const actualColumnsComparator = columnData.actualColumns;
    const [filterColumns] = (0, useDerivedState_1.useDeepDerivedState)(() => {
        var _a;
        let transformedFilters = (0, transformFilter_1.transformTableFiltersToColumns)((_a = props.filters) !== null && _a !== void 0 ? _a : []);
        return [
            ...columnData.actualColumns,
            ...transformedFilters
        ];
    }, [props.filters, actualColumnsComparator]);
    const canEdit = typeof props.onSaveQuickEdit === 'function' && columnData.hasEditor && (columnData.primaryKeyCount === 1 || typeof props.getRowKey === 'function');
    const canSelectRows = !!props.canSelectRows && (columnData.primaryKeyCount === 1 || typeof props.getRowKey === 'function');
    const [pagination, setPagination] = (0, useParsedQS_1.useParsedQs)({ page: 1, perPage: (_d = (_c = props.paginateOptions) === null || _c === void 0 ? void 0 : _c.defaultPerPage) !== null && _d !== void 0 ? _d : 10 }, (qsPagination) => {
        var _a, _b, _c, _d;
        return ({
            page: (_a = qsPagination.page) !== null && _a !== void 0 ? _a : 1,
            perPage: (_d = (_b = qsPagination.perPage) !== null && _b !== void 0 ? _b : (_c = props.paginateOptions) === null || _c === void 0 ? void 0 : _c.defaultPerPage) !== null && _d !== void 0 ? _d : 10,
        });
    }, (state) => state, Object.assign({}, props.qs));
    const [searchQuery, setSearchQuery] = (0, use_querystate_1.useQueryState)({ query: '' }, Object.assign({}, props.qs));
    const { defaultRawFilter, defaultConvertedFilter } = (0, react_1.useMemo)(() => {
        if (typeof defaultFilter === 'string' || typeof defaultFilter === 'undefined') {
            if (!(defaultFilter === null || defaultFilter === void 0 ? void 0 : defaultFilter.trim())) {
                return { defaultRawFilter: {}, defaultConvertedFilter: { groupOperator: 'and', filters: [] } };
            }
            // parse the filter string like query
            let parsedDefaultRawFilter = querystring_1.QueryString.parse(defaultFilter);
            parsedDefaultRawFilter = Object.keys(parsedDefaultRawFilter).length ? { filter: parsedDefaultRawFilter } : {};
            return {
                defaultRawFilter: parsedDefaultRawFilter,
                defaultConvertedFilter: (0, transformFilter_1.convertFromQS)(parsedDefaultRawFilter, filterColumns),
            };
        }
        const defaultRawFilter = { filter: defaultFilter };
        return {
            defaultRawFilter,
            defaultConvertedFilter: (0, transformFilter_1.convertFromQS)(defaultRawFilter, filterColumns),
        };
    }, [defaultFilter, filterColumns]);
    const [rawFilter, setRawFilter] = (0, use_querystate_1.useQueryState)(defaultRawFilter, Object.assign(Object.assign({}, props.qs), { types: {
            filter: 'any'
        }, filterToTypeDef: true }));
    const [filter, setFilter] = (0, useParsedQS_1.useParsedQs)(defaultConvertedFilter, (qsFilter) => (0, transformFilter_1.convertFromQS)(qsFilter, filterColumns), (state) => (0, transformFilter_1.convertToQS)(state, filterColumns), Object.assign(Object.assign({}, props.qs), { types: {
            filter: 'any'
        }, filterToTypeDef: true }));
    const [columnSort, setColumnSort] = (0, useParsedQS_1.useParsedQs)({ sort: (_e = props.defaultSort) !== null && _e !== void 0 ? _e : [] }, (qsSort) => {
        var _a;
        return ({
            sort: ((_a = qsSort === null || qsSort === void 0 ? void 0 : qsSort.sort) !== null && _a !== void 0 ? _a : []).map(v => {
                let parts = v.split(' ').filter(a => !!a);
                if (parts.length !== 2)
                    return null;
                return {
                    column: parts[0],
                    direction: (parts[1].toLowerCase() === 'desc' ? 'desc' : 'asc')
                };
            })
                .filter(comparators_1.notEmpty)
        });
    }, (state) => ({
        sort: state.sort.map(v => `${v.column} ${v.direction}`)
    }), Object.assign(Object.assign({}, props.qs), { types: {
            sort: 'string[]'
        } }));
    const setCtxData = (0, contexts_1.useTableContexSetter)();
    const SearchForm = (_g = (_f = props.components) === null || _f === void 0 ? void 0 : _f.SearchForm) !== null && _g !== void 0 ? _g : search_1.SearchForm;
    const ActionButtons = (_j = (_h = props.components) === null || _h === void 0 ? void 0 : _h.ActionButtons) !== null && _j !== void 0 ? _j : actions_1.TableActionButtons;
    const DataProvider = (_l = (_k = props.components) === null || _k === void 0 ? void 0 : _k.DataProvider) !== null && _l !== void 0 ? _l : data_provider_1.TableDataProvider;
    const TableWrapper = (_o = (_m = props.components) === null || _m === void 0 ? void 0 : _m.TableWrapper) !== null && _o !== void 0 ? _o : 'div';
    let wrapperStyle = {
        '--ts-dt-fixed-bg': (_p = props.fixedColBg) !== null && _p !== void 0 ? _p : 'white'
    };
    if (typeof ((_q = columnData.preMDRColumn) === null || _q === void 0 ? void 0 : _q.preMDRColumnWidth) === 'number') {
        wrapperStyle['--premdr-width'] = `${columnData.preMDRColumn.preMDRColumnWidth}px`;
    }
    const topEl = (0, react_1.useRef)(null);
    const theadEl = (0, react_1.useRef)(null);
    const tableWrapperEl = (0, react_1.useRef)(null);
    // The resize here is to check if the top area is to small and should wrap
    // page/action buttons vs. search/filter bar
    (0, react_1.useEffect)(() => {
        function topResize() {
            var _a, _b;
            let width = (_b = (_a = topEl.current) === null || _a === void 0 ? void 0 : _a.offsetWidth) !== null && _b !== void 0 ? _b : 99999;
            if (width < 800) {
                topEl.current.classList.add('wrap');
            }
            else {
                topEl.current.classList.remove('wrap');
            }
        }
        window.addEventListener('resize', topResize);
        topResize();
        return () => {
            window.removeEventListener('resize', topResize);
        };
    }, []);
    // On page change - ensure scrolled to top (if enabled)
    let fixedHeaders = props.fixedHeaders;
    let scrollToTopEnabled = !((_r = props.paginateOptions) === null || _r === void 0 ? void 0 : _r.disableScrollToTop);
    let scrollToTopLoaded = (0, react_1.useRef)(false);
    (0, react_1.useEffect)(() => {
        var _a, _b, _c;
        /**
         * Need both of these depending on the UI styling.
         *
         * The two scenarios depend on page styling and up to developer:
         *   1. Bottom actions and scrollbar always visible (ideal)
         *   2. Whole page scrolls
         *
         * For scenario 1 - scrolling to topEl (search/action buttons)
         * doesn't really work as they are also always visible so we need
         * to scroll to show thead (which also breaks down for fixed headers,
         * but that isn't supported yet).
         *
         * For scenario 2 - scrolling to thead, while it would work, probably
         * isn't ideal, as it's nice to get the top page nav and action buttons
         * available.
         *
         * We can't detect these scenarios - so we do both.
         * thead first, then a topEl and the topEl one might not scroll
         *
         * If we did it the other way - then it _might_ scroll back down
         * and hide part of topEl.
         */
        if (scrollToTopLoaded.current && scrollToTopEnabled) {
            if (fixedHeaders !== null && fixedHeaders !== void 0 ? fixedHeaders : true) {
                // headers are fixed, it's likely they are in view already - just scroll the wrapper
                (_a = tableWrapperEl.current) === null || _a === void 0 ? void 0 : _a.scrollTo({ top: 0 });
            }
            else {
                // headers are not fixed we should scroll them into view
                (_b = theadEl.current) === null || _b === void 0 ? void 0 : _b.scrollIntoView();
                (_c = topEl.current) === null || _c === void 0 ? void 0 : _c.scrollIntoView();
            }
        }
        scrollToTopLoaded.current = true;
    }, [pagination, scrollToTopEnabled, scrollToTopLoaded, fixedHeaders]);
    function doSetSelectedRows(selection) {
        if (typeof props.onSelectionChange === 'function') {
            // raise
            let keys = Object.keys(selection);
            let values = Object.values(selection);
            props.onSelectionChange(keys, values);
        }
        setCtxData({ selectedRows: selection });
    }
    function setAllSelected(selectAll, data) {
        let newSelected = {};
        if (selectAll) {
            data.map((row, idx) => {
                let rowKey = (0, getRowKey_1.getRowKey)(row, idx, columnData.actualColumns, props.getRowKey);
                newSelected[rowKey] = row;
                return row;
            });
        }
        doSetSelectedRows(newSelected);
    }
    function setRowSelected(row, rowIndex) {
        let rowKey = (0, getRowKey_1.getRowKey)(row, rowIndex, columnData.actualColumns, props.getRowKey);
        setCtxData(prev => {
            const exists = typeof prev.selectedRows[rowKey] !== 'undefined';
            let newState = prev;
            if (!exists) {
                newState = (0, immutable_1.update)(prev, {
                    selectedRows: {
                        [rowKey]: { $set: row },
                    }
                });
            }
            else {
                newState = (0, immutable_1.update)(prev, {
                    selectedRows: {
                        $unset: [rowKey]
                    }
                });
            }
            if (typeof props.onSelectionChange === 'function') {
                // raise
                let keys = Object.keys(newState.selectedRows);
                let values = Object.values(newState.selectedRows);
                props.onSelectionChange(keys, values);
            }
            return newState;
        });
    }
    const searchFormOnSearch = (0, react_1.useCallback)((query) => {
        (0, use_querystate_1.batchedQSUpdate)(() => {
            setSearchQuery({ query });
            setPagination(prev => ({ page: 1, perPage: prev.perPage }));
        });
    }, [setSearchQuery, setPagination]);
    const searchFormApplyFilter = (0, react_1.useCallback)((newFilter) => {
        (0, use_querystate_1.batchedQSUpdate)(() => {
            setRawFilter({ filter: newFilter });
            setPagination(prev => ({ page: 1, perPage: prev.perPage }));
        });
    }, [setRawFilter, setPagination]);
    const actualColumnSorts = (0, react_1.useMemo)(() => {
        return [
            ...groupBy,
            ...columnSort.sort.filter(s => (!groupBy.length || !groupBy.find(g => g.column === s.column))),
        ];
    }, [groupBy, columnSort]);
    (0, react_1.useImperativeHandle)(methodRef, () => ({
        clearSelection: () => {
            doSetSelectedRows({});
        },
        getState: () => {
            return {
                filter,
                query: searchQuery.query,
                sort: columnSort.sort,
                columnConfig: {
                    columnOrder,
                    visibility: columnVisibility,
                    groupBy: qsGroupBy.group
                }
            };
        },
        setState: (value) => {
            (0, use_querystate_1.batchedQSUpdate)(() => {
                if (value.filter) {
                    setFilter(value.filter);
                }
                if (value.query) {
                    setSearchQuery({ query: value.query });
                }
                if (value.sort) {
                    setColumnSort({ sort: value.sort });
                }
                if (value.columnConfig) {
                    react_dom_1.default.unstable_batchedUpdates(() => {
                        setColumnVisibility(value.columnConfig.visibility);
                        setColumnOrder(value.columnConfig.columnOrder);
                        setGroupBy({ group: value.columnConfig.groupBy });
                    });
                }
                setPagination(prev => ({ page: 1, perPage: prev.perPage }));
            });
        },
    }), [
        doSetSelectedRows,
        filter, setFilter,
        searchQuery, setSearchQuery,
        columnSort, setColumnSort,
        columnVisibility, setColumnVisibility,
        columnOrder, setColumnOrder,
        qsGroupBy, setGroupBy
    ]);
    let tableContainerClasses = ['ts-datatable', 'ts-datatable-container'];
    if ((_s = props.tableContainerProps) === null || _s === void 0 ? void 0 : _s.className)
        tableContainerClasses.push((_t = props.tableContainerProps) === null || _t === void 0 ? void 0 : _t.className);
    if (((_u = props.fixedHeaders) !== null && _u !== void 0 ? _u : true))
        tableContainerClasses.push('fixed-headers');
    if (((_v = props.fixedFooters) !== null && _v !== void 0 ? _v : false))
        tableContainerClasses.push('fixed-footers');
    /**
     * Finally we setup the contexts that will house all the data
     * and pass it to all the subcomponents for eventual display.
     */
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(contexts_1.ColumnContext.Provider, { value: Object.assign(Object.assign({}, columnData), { filterColumns,
                canGroupBy, columnSorts: actualColumnSorts, groupBy: groupBy, multiColumnSorts: (_w = props.multiColumnSorts) !== null && _w !== void 0 ? _w : false, filter, filterSettings: props.filterSettings, canSelectRows,
                setFilter,
                setColumnSort,
                setAllSelected,
                setRowSelected, onShowColumnPicker: props.onShowColumnPicker, onShowFilterEditor: props.onShowFilterEditor, setPagination, getRowKey: props.getRowKey, propOnSaveQuickEdit: props.onSaveQuickEdit, DetailRow: props.DetailRow, canRowShowDetail: props.canRowShowDetail, columnOrder,
                setColumnConfig, canReorderColumns: (_x = props.canReorderColumns) !== null && _x !== void 0 ? _x : false, classNames: props.classNames, labels: props.labels, components: props.components, canSelectRow: props.canSelectRow, groupsExpandedByDefault: (_y = props.groupsExpandedByDefault) !== null && _y !== void 0 ? _y : true, doNotUseHTML5Dialog: props.doNotUseHTML5Dialog, doNotUseRefetchAfterSave: (_z = props.doNotUseRefetchAfterSave) !== null && _z !== void 0 ? _z : false, getTableRowProps: props.getTableRowProps, getTableCellProps: props.getTableCellProps }) },
            react_1.default.createElement(DataProvider, { TableBody: data_provider_1.DataProviderContent, passColumnsToQuery: passColumnsToQuery, pagination: pagination, searchQuery: hideSearchForm ? '' : ((_0 = searchQuery.query) !== null && _0 !== void 0 ? _0 : ''), onQueryChange: props.onQueryChange, data: props.data, totalCount: props.totalCount, isLoading: props.isLoading, footerData: props.footerData, refetch: props.refetch, canGroupBy: canGroupBy, filters: filter, sorts: actualColumnSorts },
                react_1.default.createElement("div", Object.assign({ id: props.id, style: wrapperStyle }, ((_1 = props.tableContainerProps) !== null && _1 !== void 0 ? _1 : {}), { className: tableContainerClasses.join(' ') }),
                    react_1.default.createElement("div", { ref: topEl, className: `ts-datatable-top` },
                        react_1.default.createElement("div", { className: 'ts-datatable-search-filters' },
                            !hideSearchForm && react_1.default.createElement(SearchForm, { searchQuery: (_2 = searchQuery.query) !== null && _2 !== void 0 ? _2 : '', filter: rawFilter.filter, onSearch: searchFormOnSearch, applyFilter: searchFormApplyFilter }),
                            react_1.default.createElement(filter_1.FilterBar, null)),
                        react_1.default.createElement("div", { className: 'ts-datatable-page-actions' },
                            react_1.default.createElement("div", { className: "ts-datatable-actions" },
                                react_1.default.createElement(ActionButtons, { position: 'top', quickEditPosition: quickEditPosition, buttons: {
                                        quickEdit: react_1.default.createElement(editButton_1.TableEditorButton, { canEdit: canEdit }),
                                        filter: react_1.default.createElement(filter_1.FilterButton, null),
                                        columnPicker: react_1.default.createElement(column_picker_1.ColumnPickerButton, null),
                                        refresh: react_1.default.createElement(actions_1.TableRefreshButton, { hideRefetch: props.hideRefetch }),
                                    } })),
                            (paginate === 'top' || paginate === 'both') &&
                                react_1.default.createElement(pagination_1.Paginate, Object.assign({}, props.paginateOptions, pagination, { changePage: (page) => setPagination(prev => (Object.assign(Object.assign({}, prev), page))), PaginateComponent: (_3 = props.components) === null || _3 === void 0 ? void 0 : _3.Paginate })))),
                    react_1.default.createElement(TableWrapper, Object.assign({ id: `ts-datatable-wrapper_${props.id}`, ref: tableWrapperEl }, ((_4 = props.tableWrapperProps) !== null && _4 !== void 0 ? _4 : {}), { className: `ts-datatable-wrapper ${(_6 = (_5 = props.tableWrapperProps) === null || _5 === void 0 ? void 0 : _5.className) !== null && _6 !== void 0 ? _6 : ''}` }),
                        react_1.default.createElement("table", Object.assign({}, ((_7 = props.tableProps) !== null && _7 !== void 0 ? _7 : {}), { className: `ts-datatable-table ${(_9 = (_8 = props.tableProps) === null || _8 === void 0 ? void 0 : _8.className) !== null && _9 !== void 0 ? _9 : ''}` }),
                            react_1.default.createElement(header_1.TableHeader, { headRef: theadEl, enableColumnSearch: !!props.enableColumnSearch }),
                            react_1.default.createElement(body_1.TableBody, { getRowKey: props.getRowKey, canEditRow: props.canEditRow, LoadingComponent: (_10 = props.components) === null || _10 === void 0 ? void 0 : _10.Loading }),
                            react_1.default.createElement(footer_1.TableFooter, null))),
                    (paginate === 'bottom' || paginate === 'both') &&
                        react_1.default.createElement("div", { className: 'ts-datatable-bottom-page' },
                            react_1.default.createElement(pagination_1.Paginate, Object.assign({}, props.paginateOptions, pagination, { changePage: (page) => setPagination(prev => (Object.assign(Object.assign({}, prev), page))), PaginateComponent: (_11 = props.components) === null || _11 === void 0 ? void 0 : _11.Paginate }))),
                    react_1.default.createElement("div", { className: "ts-datatable-bottom-actions" },
                        react_1.default.createElement(ActionButtons, { position: 'bottom', quickEditPosition: quickEditPosition, buttons: {
                                quickEdit: react_1.default.createElement(editButton_1.TableEditorButton, { canEdit: canEdit }),
                                filter: react_1.default.createElement(filter_1.FilterButton, null),
                                columnPicker: react_1.default.createElement(column_picker_1.ColumnPickerButton, null),
                                refresh: react_1.default.createElement(actions_1.TableRefreshButton, { hideRefetch: props.hideRefetch }),
                            } })))))));
};
/**
 * Basic Structure
 * <full container>
 *    <section>
 *      <form>
 *      <actions>
 *    <section>
 *      <filter>
 *      <pages>
 *    <table-wrapper>
 *      <table>
 *        <table-header>
 *        <table-body>
 *    <section>
 *      <actions>
 *    <section>
 *      <pages>
 */
const DataTable = function DataTable(_a) {
    var { editMode } = _a, props = __rest(_a, ["editMode"]);
    return (react_1.default.createElement(contexts_2.TableContextProvider, { editMode: editMode !== null && editMode !== void 0 ? editMode : 'default' },
        react_1.default.createElement(DataTableCore, Object.assign({}, props))));
};
exports.DataTable = DataTable;
