"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSaveQuickEdit = void 0;
const react_1 = require("react");
const getRowKey_1 = require("../../utils/getRowKey");
const contexts_1 = require("./contexts");
const data_provider_1 = require("./data-provider");
function useSaveQuickEdit() {
    const { data: currentData, refetch } = (0, react_1.useContext)(data_provider_1.TableDataContext);
    const { actualColumns, getRowKey: propGetRowKey, propOnSaveQuickEdit, doNotUseRefetchAfterSave } = (0, react_1.useContext)(contexts_1.ColumnContext);
    const setCtxData = (0, contexts_1.useTableContexSetter)();
    const onSaveQuickEdit = (0, react_1.useCallback)((data) => __awaiter(this, void 0, void 0, function* () {
        try {
            if (!!propOnSaveQuickEdit && Object.keys(data).length) {
                setCtxData({ isSavingQuickEdit: true });
                let rowsToSave = Object.keys(data);
                let originalData = {};
                for (let id of rowsToSave) {
                    let rowData = currentData.find((row, idx) => (0, getRowKey_1.getRowKey)(row, idx, actualColumns, propGetRowKey) == id);
                    originalData[id] = rowData;
                }
                yield propOnSaveQuickEdit(data, originalData);
                setCtxData({ editData: {} });
                if (!doNotUseRefetchAfterSave) {
                    refetch === null || refetch === void 0 ? void 0 : refetch();
                }
            }
            setCtxData({ isEditing: false });
        }
        catch (_a) {
            // avoid unhandled exception
        }
        finally {
            setCtxData({ isSavingQuickEdit: false });
        }
    }), [propOnSaveQuickEdit, currentData, actualColumns, setCtxData, propGetRowKey]);
    return onSaveQuickEdit;
}
exports.useSaveQuickEdit = useSaveQuickEdit;
