"use strict";
// Adapted from: https://hackernoon.com/whats-the-right-way-to-fetch-data-in-react-hooks-a-deep-dive-2jc13230
Object.defineProperty(exports, "__esModule", { value: true });
exports.useArrayDerivedState = exports.useDerivedState = exports.useDeepDerivedState = void 0;
const react_1 = require("react");
const comparators_1 = require("./comparators");
function useDeepDerivedState(onDepChange, depList) {
    let value = useCommonDerivedState(onDepChange, depList, comparators_1.deepCompare);
    (0, react_1.useDebugValue)(value[0]);
    return value;
}
exports.useDeepDerivedState = useDeepDerivedState;
function useDerivedState(onDepChange, depList) {
    let value = useCommonDerivedState(onDepChange, depList, comparators_1.simpleCompare);
    (0, react_1.useDebugValue)(value[0]);
    return value;
}
exports.useDerivedState = useDerivedState;
function useArrayDerivedState(onDepChange, depList) {
    let value = useCommonDerivedState(onDepChange, depList, comparators_1.arrayCompare);
    (0, react_1.useDebugValue)(value[0]);
    return value;
}
exports.useArrayDerivedState = useArrayDerivedState;
function useCommonDerivedState(onDepChange, depList, comparator) {
    const [, setRerender] = (0, react_1.useState)(1);
    const localRef = (0, react_1.useRef)({ init: false });
    let currPublicState;
    if (!localRef.current.init || !comparator(depList, localRef.current.depList)) {
        currPublicState = onDepChange(!localRef.current.init ? null : localRef.current.publicState);
        localRef.current = {
            init: true,
            publicState: currPublicState,
            depList
        };
    }
    else {
        currPublicState = localRef.current.publicState;
    }
    const publicSetState = (0, react_1.useCallback)((newState) => {
        if (!localRef.current.init)
            throw new Error();
        const publicState = typeof newState === 'function'
            ? newState(localRef.current.publicState)
            : newState;
        localRef.current = Object.assign(Object.assign({}, localRef.current), { publicState });
        setRerender(v => 0 - v); // toggle's between 1 and -1
    }, [localRef]);
    return [currPublicState, publicSetState];
}
