"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLocalState = void 0;
const react_1 = require("react");
const storage_1 = require("./storage");
const comparators_1 = require("./comparators");
function useLocalState(key, defaultValue, depList) {
    var _a;
    const [, setRerender] = (0, react_1.useState)(1);
    const localRef = (0, react_1.useRef)({ init: false });
    let currPublicState;
    if (!localRef.current.init || !(0, comparators_1.simpleCompare)(depList, localRef.current.depList)) {
        currPublicState = (_a = (0, storage_1.getLocalState)(key)) !== null && _a !== void 0 ? _a : defaultValue;
        localRef.current = {
            init: true,
            publicState: currPublicState,
            depList
        };
    }
    else {
        currPublicState = localRef.current.publicState;
    }
    const publicSetState = (0, react_1.useCallback)((newState) => {
        if (!localRef.current.init)
            throw new Error();
        const publicState = typeof newState === 'function'
            ? newState(localRef.current.publicState)
            : newState;
        (0, storage_1.saveLocalState)(key, publicState);
        localRef.current = Object.assign(Object.assign({}, localRef.current), { publicState });
        setRerender(v => 0 - v); // toggle's between 1 and -1
    }, [key, localRef]);
    return [currPublicState, publicSetState];
}
exports.useLocalState = useLocalState;
